% Compute joint and posterior probability of selection given allele frequency
%
% Input:
% s - possible values of selection coefficient
% beta - possible values of effect size
% N - effective population size
% coupling_params - distribution-specific parameters structure
% coupling_str - type of joint distribution of effect size and selection coefficient
%
% Output:
% g_dist - joint distribution of effect size and selection coefficient
% s_hist - marginal distribution of selection coefficient
% beta_hist - marginal distribution of effect size
%
function [g_hist s_hist beta_hist] = joint_selection_effect_size_allele_freq(s, beta, ... % allele frequency distribution
    N, coupling_str, coupling_params) % effective population size


if(isvector(s))
    num_s = length(s); num_beta = length(beta);
    [s_mesh, beta_mesh] = meshgrid(s, beta); % create matrices
else
    num_s = size(s,2); num_beta = size(s,1);
end

S = 4.*N.*s; % compute big S
g_hist = zeros(num_s, num_beta); % Get space for the joint distribution

% Compute weighted average of Hartl distribution, with weights given by the selection coefficient distirbution
switch lower(coupling_str) % allow different relationships between effect size and selection coefficient
    case {'eyre_walker', 'eyre-walker'} % use relationship from Eyre-Walker pnas 2010
        s_hist = gampdf(-S(1,:), coupling_params.k, coupling_params.theta / coupling_params.k); % Gamma distribution for marginal s. Use Big S here !!! 
        mu = (-s) .^ coupling_params.tau; % expected beta for each s 
        for i=1:num_s % Now compute the conditional distribution of beta given s
            g_hist(i,:) = s_hist(i) * 0.5 .* (...
                normpdf(beta(:,i), mu(i), mu(1,i) * coupling_params.sigma) + ...
                normpdf(beta(:,i), -mu(i), mu(1,i) * coupling_params.sigma))'; % compute a Gaussian around mean (symmetric)
        end
    case 'balancing' % assuming a balancing selection model
        beta_hist = normpdf(beta, 0, 1); % assume standard Gaussian
        s_of_beta = exp(-beta.^2 ./ (2.*(1+coupling_params.sigma^2))); % set deterministic relation
        for i=1:num_beta
            [~, cur_s_ind] = min(abs(s - s_of_beta(i))); % Find closest index
            g_hist(cur_s_ind,i) = beta_hist(i);
        end
    case 'linear' % linear relationship between s and beta
        slope = coupling_params;
        if(isvector(s))
            for i=1:num_s
                [~, j] = min(abs(beta - s(i)));
                g_hist(i,j) = 1;
            end
        else
            s_of_beta = [beta -beta] .*slope;
        end
        
    case 'sigmoid'
        slope = coupling_params;
        offset = 0.5; % slope = 30;
        s_of_beta = -0.2+0.2.*[ 1 - 1 ./ (1+exp(-(((beta)-offset).*slope)))  1 - 1 ./ (1+exp(-(((-beta)-offset).*slope)))]; %  1 ./ (1+exp(abs((beta-0.4).*slope)))-0.5];
        %        figure; plot(beta(:,1), s_of_beta(:,1), '.');   xlabel('\beta'); ylabel('s'); title('Sigmoid Curve');
        
    case 'quadratic'
        
    case 'uniform' % no coupling
        g_hist(:) = 1;
        
    case 'simple_eric' % simple case: 2 classes of mutations
        s_hist(1) = 1 - coupling_params.alpha_null; % the proportion of mutations that are harmless
        [~, cur_s_ind] = min(abs(s - coupling_params.null_s)); % Find closest index
        s_hist(cur_s_ind) = coupling_params.alpha_null; % null mutations
        
        
        for i = [1 cur_s_ind] % no need to loop on everything
            switch i
                case 1
                    [~, cur_beta_ind] = min(abs(beta - coupling_params.null_beta)); % Find closest index
                otherwise
            end
            g_hist(i,cur_beta_ind) = s_hist(i); % set deterministic beta for each s
        end % try different selection coefficients
        
end % switch coupling type

if(max(g_hist(:)) == 0)  % force s=beta 
    epsilon = 0.000001;
    [~, I] = min(abs(s_of_beta(:,1:num_s)-s));
    if(size(s_of_beta, 2) > num_s)
        [~, J] = min(abs(s_of_beta(:,num_s+1:end)-s)); % (end:-1:1,:)- s)); % here flip beta and minus-beta (assume symmetry with respect to beta)
        diag_inds = union( sub2ind([num_s num_beta], 1:num_s, I), sub2ind([num_s num_beta], 1:num_s, J) );
    else
        diag_inds = sub2ind([num_s num_beta], 1:num_s, I);
    end
    more_diag_inds = find(abs(s_of_beta(:,1:num_s)-s) < epsilon);
    diag_inds = union(diag_inds, more_diag_inds);
    
    g_hist(diag_inds) = 1; 
end

g_hist = normalize_hist2d(s, beta, g_hist); % normalize histogram to integrate to one
s_hist = sum(g_hist,2)'; beta_hist = sum(g_hist); % get marginal distributions
s_hist = normalize_hist(s(1,:), s_hist);  % normalize marginal distributions 
beta_hist = normalize_hist(beta(:,1)', beta_hist);
g_hist = g_hist'; % transpose 

