% Simulate phenotype according to LP model
% 
% Input: 
% SNP_mat - matrix of SNP values for each SNP and each individuals
% f_vec - vector of allele frequencies 
% num_pathways - in the LP model 
% h_pathway - heritability of each pathway
% h_shared_env - shared envirounment of each pathway (not used)
% trait_type - quantitative or binary (disease)
% mu - prevalence for disease trait 
%
% Output: 
% LP_phenotype_vec - phenotype vector
% LP_max_pathway_vec - which pathway of the num_pathway gave the maximum liability
function [LP_phenotype_vec LP_max_pathway_vec] = simulate_LP_phenotypes(SNP_mat, f_vec, num_pathways, h_pathway, h_shared_env, trait_type, mu)

[num_snps, num_individuals] = size(SNP_mat);

[snp_start_inds snp_end_inds] = ...
    divide_region_to_blocks(1, num_snps, ceil(num_snps/num_pathways));



beta_snps = sqrt(num_pathways / sum(f_vec .* (1-f_vec))); % set coefficients such that all SNPs have same variance explained and total variance sums to one
beta_snps_vec = sqrt(num_pathways ./ (num_snps.* f_vec .* (1-f_vec))); % set coefficients such that all SNPs have same variance explained and total variance sums to one

SNP_mat = SNP_mat - repmat(f_vec, 1, num_individuals); % normalize genotypes to have mean zero
% New: normalize all SNPs and set equal effect size
% SNP_mat = SNP_mat - repmat(mean(SNP_mat,2), 1, num_individuals); 
% SNP_mat = SNP_mat ./ repmat(std(SNP_mat,[],2), 1, num_individuals); 
% beta_snps = sqrt(num_pathways / num_snps); 


LP_phenotype_vec = zeros(num_individuals,1)-99999999999; % start with very negative
LP_max_pathway_vec = zeros(num_individuals,1); % start with no index 
for i=1:num_pathways
    temp_pathway_vec = ...
        sum(repmat(beta_snps_vec(snp_start_inds(i):snp_end_inds(i)), 1, num_individuals) .* ...
        SNP_mat(snp_start_inds(i):snp_end_inds(i),:))' .* sqrt(h_pathway) + ...
        randn(num_individuals,1) .* sqrt(1-h_pathway);% Generate phenotype
    LP_phenotype_vec = max(LP_phenotype_vec, temp_pathway_vec);
    temp_max_inds = find(temp_pathway_vec == LP_phenotype_vec); 
    LP_max_pathway_vec(temp_max_inds) = i; % take index of maximum pathway
end

[mu_z sigma_z] = maxnormstat(num_pathways); 
switch trait_type
    case {'disease', 'binary'}
        x_mu = norminv(1-mu); % set threshold for disease
        LP_phenotype_vec = LP_phenotype_vec > x_mu;

    case 'quantitative'
        LP_phenotype_vec = (LP_phenotype_vec - mu_z) ./ sigma_z; % Normalize 
end


